--[[
AutoProfitY - Automatically sells low quality gray items and selected white items to the vendor.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:
	
Free Software Foundation, I.,
51 Franklin Street, Fifth Floor,
Boston, MA  02110-1301, USA.
--]]

local MAJOR = 'AutoProfitY'
local Addon = LibStub( 'AceAddon-3.0'):NewAddon( MAJOR, 'AceEvent-3.0', 'AceConsole-3.0', 'LibDebugLog-1.0')
if not Addon then return end -- already loaded and no upgrade necessary

local L    = LibStub( 'AceLocale-3.0'):GetLocale( MAJOR)
local LibG = LibStub( 'LibGratuity-3.0')

Addon.name, Addon.localizedname = GetAddOnInfo( MAJOR)
Addon.version = format( '%s: %s |cFF20FF20(Cata)|r', L.Version, GetAddOnMetadata( MAJOR, 'Version') or 'unknown')

local DEFAULTS = { 
	profile = {
		enabled    = true,
		autosell   = false,
		silent     = false,
		showtotal  = true,
		limit      = true,
		soulbound  = false,
		spin       = 'profit',
		exceptions = {}
	},
	realm = {
		exceptions = {}
	}
}
local CLASS_MATCH = ITEM_CLASSES_ALLOWED:format( '([%w%s,]*)')
local PROFICIENCIES = {}
local DEFAULT_SPIN_RATE = 0.6

local button = LibStub( 'AceGUI-3.0'):Create( 'ProfitButton') -- ProfitModel, ProfitButton
local rotation = 0
local rotrate = 0
local options = {
	type = 'group', name = Addon:GetName(), childGroups = 'tree', args = {}
}

--[[---------------------------------------------------------------------------------
Local
------------------------------------------------------------------------------------]]
local function SetRate( rate)
	rotation = 0
	rotrate = rate or DEFAULT_SPIN_RATE
	Addon:Debug( 'SetRate', rotrate)
end

local function IsSpin( spin)
	return Addon.db.profile.spin == spin
end

local function Coins( money)
	money = math.floor( tonumber( money) or 0)
	local g = math.floor( money / 10000)
	local s = math.floor( money % 10000 / 100)
	local c = money % 100
	if g > 0 then
		return format( '|cFFFFD100%dg|cFF000000.|cFFE6E6E6%02ds|cFF000000.|cFFC8602C%02dc|r', g, s, c)
	end
	if s > 0 then
		return format( '|cFFE6E6E6%ds|cFF000000.|cFFC8602C%02dc|r', s, c)
	end
	return format( '|cFFC8602C%dc|r', c)
end

local function IsSoulboundJunk( bagID, slotID, link)
	if not Addon.db.profile.soulbound then
		return false 
	end
	Addon:Debug( 'IsSoulboundJunk', link)
	LibG:SetBagItem( bagID, slotID)
	if not LibG:Find( ITEM_SOULBOUND) then
		return false
	end
	Addon:Debug( 'IsSoulboundJunk', 'soulbound')
	local _, _, classes = LibG:Find( CLASS_MATCH)
	if classes then
		Addon:Debug( 'IsSoulboundJunk', 'class match', classes)
		local _, class = UnitClass( 'player')
		local male = LOCALIZED_CLASS_NAMES_MALE[class]
		if not classes:match( male) then
			return true
		end
	end
	Addon:Debug( 'IsSoulboundJunk', 'type check')
	local _, _, _, _, _, iType, iSubType, _, iEquipLoc = GetItemInfo( link)
	local tbl = PROFICIENCIES[iType]
	return tbl and (tbl[iSubType] or (tbl.noOffhand and iEquipLoc == 'INVTYPE_WEAPONOFFHAND'))
end

local function IsJunk( link)
	local _, _, quality = GetItemInfo( link)
	local id = link:match( 'item:(%d+)')
	if Addon.db.profile.exceptions[id] then
		return quality ~= 0
 	end
	if Addon.db.realm.exceptions[id] then
		return quality ~= 0
 	end
	return quality == 0
end

local function GetItemProfit( bagID, slotID, link)
	local _, itemCount = GetContainerItemInfo( bagID, slotID)
	local _, _, _, _, _, _, _, _, _, _, itemSellPrice = GetItemInfo( link)
	return itemCount * (tonumber( itemSellPrice) or 0)
end

local function GetProfit()
	local profit = 0
	for bagID = 0,NUM_BAG_SLOTS do
		for slotID = 1,GetContainerNumSlots( bagID) do
			local link = GetContainerItemLink( bagID, slotID)
			if link and (IsJunk( link) or IsSoulboundJunk( bagID, slotID, link)) then
				profit = profit + GetItemProfit( bagID, slotID, link)
			end
		end
	end
	return profit
end

local function SellJunk()
	local profit = 0
	local items = 0
	for bagID = 0,NUM_BAG_SLOTS do
		for slotID = 1,GetContainerNumSlots( bagID) do
			local link = GetContainerItemLink( bagID, slotID)
			if link and (IsJunk( link) or IsSoulboundJunk( bagID, slotID, link)) then
				items = items + 1
				profit = profit + GetItemProfit( bagID, slotID, link)
				UseContainerItem( bagID, slotID)
				if Addon.db.profile.report then
					Addon:Print( L.Sold:format( link))
				end
			end
			if Addon.db.profile.limit and items >= 12 then break end
		end
		if Addon.db.profile.limit and items >= 12 then break end
	end
	if profit > 0 and Addon.db.profile.profit then
		Addon:Print( L.Total:format( Coins( profit)))
	end
end

--[[---------------------------------------------------------------------------------
Script
------------------------------------------------------------------------------------]]
local function Button_OnEnter( button)
	Addon:Debug( 'Button_OnEnter')
	GameTooltip:SetOwner( button.frame, 'ANCHOR_RIGHT')
	GameTooltip:SetText( L.SellJunk)
	if IsSpin( 'over') then
		SetRate()
	elseif IsSpin( 'overProfit') then
		local profit = GetProfit()
		if profit > 0 then
			SetRate()
			SetTooltipMoney( GameTooltip, profit, nil, L.ExpectProfit)
		else
			GameTooltip:AddLine( L.NoJunk, 1.0, 1.0, 1.0, 1)
		end
	end
	GameTooltip:Show()
end

local function Button_OnLeave( button)
	Addon:Debug( 'Button_OnLeave')
	GameTooltip:Hide()
	if IsSpin( 'over') then
		SetRate( 0)
	end
end

local function Button_OnClick( button, event)
	if MerchantFrame.selectedTab == 1 then
		GameTooltip:Hide()
		SellJunk()
		if IsSpin( 'profit') or IsSpin( 'overProfit') then
			SetRate( 0)
		end
	end
end

local function Button_OnUpdate( frame, elapsed)
	if rotrate > 0 then
		rotation = rotation + (elapsed * rotrate)
		button:SetRotation( rotation)
	else
		button:SetRotation()
	end
end

--[[---------------------------------------------------------------------------------
Class
------------------------------------------------------------------------------------]]
function Addon:OnInitialize()
	self.db = LibStub( 'AceDB-3.0'):New( 'AutoProfitYDB', DEFAULTS, 'Default')
	button:SetWidth( 33)
	button:SetHeight( 33)
	button:SetCallback( 'OnEnter',  Button_OnEnter)
	button:SetCallback( 'OnLeave',  Button_OnLeave)
	button:SetCallback( 'OnClick',  Button_OnClick)
	LibStub( 'AceConfig-3.0'):RegisterOptionsTable( 'AutoProfitYDialog', options)
	self:RegisterChatCommand( 'profit', 'ShowDialog')
	self:RegisterChatCommand( 'pro', 'ShowDialog')
end

function Addon:OnEnable()
	self:Debug( 'OnEnable')
	self:ToggleDebugLog( false)
	self:RegisterEvent( 'MERCHANT_SHOW')
	self:RegisterEvent( 'MERCHANT_CLOSED')
	self:RegisterMessage( 'AUTO_PROFIT_UPDATE')
	self:Print( self.version, 'loaded')
end

function Addon:OnDisable()
	self:Debug( 'OnDisable')
	self:UnregisterEvent( 'MERCHANT_SHOW')
	self:UnregisterEvent( 'MERCHANT_CLOSED')
	self:UnregisterMessage( 'AUTO_PROFIT_UPDATE')
end

--[[---------------------------------------------------------------------------------
Main
------------------------------------------------------------------------------------]]
function Addon:MERCHANT_SHOW()
	self:Debug( 'MERCHANT_SHOW')
	self:AUTO_PROFIT_UPDATE()
	if self.db.profile.autosell then
		SellJunk()
	end
end

function Addon:MERCHANT_CLOSED()
	self:Debug( 'MERCHANT_CLOSED')
	button.frame:SetScript( 'OnUpdate', nil)
	SetRate( 0)
	self:UnregisterEvent( 'BAG_UPDATE' )
end

function Addon:BAG_UPDATE()
	self:Debug( 'BAG_UPDATE')
	self:AUTO_PROFIT_UPDATE()
end

function Addon:AUTO_PROFIT_UPDATE()
	self:Debug( 'AUTO_PROFIT_UPDATE')
	if self.db.profile.autosell then
		self:MERCHANT_CLOSED()
		button.frame:Hide()
	else
		button.frame:SetParent( MerchantFrame)
		button:ClearAllPoints()
		button:SetPoint( 'TOPRIGHT', MerchantFrame, 'TOPRIGHT', -41, -37)
		button:SetImage( 'Interface\\Icons\\INV_Misc_Coin_02')
--		button:SetModel( 'World\\Generic\\PassiveDoodads\\TreasurePiles\\GoldPileLarge01.m2')
		if IsSpin( 'profit') and GetProfit() > 0 then
			SetRate()
		else
			SetRate( 0)
		end
		button.frame:SetScript( 'OnUpdate', Button_OnUpdate)
		button.frame:Show()
		self:RegisterEvent( 'BAG_UPDATE' )
	end
end

function Addon:SetProficiency( prof)
	PROFICIENCIES = prof
end

function Addon:ShowDialog()
	for name,mod in self:IterateModules() do
		options.args[name] = mod.GetOptionTable and mod:GetOptionTable() or nil
	end
	LibStub( 'AceConfigDialog-3.0'):Open( 'AutoProfitYDialog')
end
