--[[
LibRule - library for item select rules

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:
	
Free Software Foundation, I.,
51 Franklin Street, Fifth Floor,
Boston, MA  02110-1301, USA.
--]]

local MAJOR, MINOR = 'LibRule-1.0', 1
local lib = LibStub:NewLibrary( MAJOR, MINOR)
if not lib then return end -- No Upgrade needed.

LibStub( 'AceConsole-3.0'):Embed( lib)
LibStub( 'LibDebugLog-1.0'):Embed( lib)

lib.embeds = lib.embeds or {} -- table containing objects LibRule is embedded in.

--[[---------------------------------------------------------------------------------
  local functions
------------------------------------------------------------------------------------]]
local qualities = {}

local function GetQuality( rarity)
	local q = qualities[rarity]
	if not q then
		q = getglobal( 'ITEM_QUALITY'..rarity..'_DESC')
		qualities[rarity] = q
	end
	return q
end

local function GetItemID( link)
	local id = link and link:match('|Hitem:(%d+):.*|h') -- '|c%s|Hitem:%i:0:0:0:0:0:0:0|h[%s]|h|r'
	if not id then
		lib:Debug( 'unknown link:', link and link:gsub( '|', '='))
	end
	return tonumber( id)
end

local function GetItemValue( link, info)
	if info == 'name' then
		local iName = GetItemInfo( link)
		return iName
	end
	if info == 'id' then
		local _, iLink = GetItemInfo( link)
		return GetItemID( iLink) or 0
	end
	if info == 'quality' then
		local _, _, iRarity = GetItemInfo( link)
		return GetQuality( iRarity or 0)
	end
	if info == 'level' then
		local _, _, _, iLevel = GetItemInfo( link)
		return iLevel or 0
	end
	if info == 'uselevel' then
		local _, _, _, _, iMinLevel = GetItemInfo( link)
		return iMinLevel or 0
	end
	if info == 'type' then
		local _, _, _, _, _, iType = GetItemInfo( link)
		return iType
	end
	if info == 'subtype' then
		local _, _, _, _, _, _, iSubType = GetItemInfo( link)
		return iSubType
	end
	if info == 'count' then
		local _, _, _, _, _, _, _, iStackCount = GetItemInfo( link)
		return iStackCount or 1
	end
	if info == 'equip' then
		local _, _, _, _, _, _, _, _, iEquipLoc = GetItemInfo( link)
		return iEquipLoc and _G[iEquipLoc]
	end
	return ''
end

local function GetRefValue( ref, info)
	local _, _, id = ref:find( 'ref@(%d+)')
	if id then
		return GetItemValue( id, info)
	end
	if info == 'id' then
		return tonumber( ref) or 0
	end
	if info == 'level' then
		return tonumber( ref) or 0
	end
	if info == 'uselevel' then
		return tonumber( ref) or 0
	end
	if info == 'count' then
		return tonumber( ref) or 1
	end
	return ref
end

--[[---------------------------------------------------------------------------------
  environment functions
-- iName, iLink, iRarity, iLevel, iMinLevel, iType, iSubType, iStackCount, iEquipLoc, iTexture, iSellPrice = GetItemInfo( link)
------------------------------------------------------------------------------------]]
local Environment = {
	['name'] = function( link, ...)
		local iName = GetItemValue( link, 'name')
		for n = 1,select( '#', ...) do
			local ref = select( n, ...)
			if iName == GetRefValue( ref, 'name') then
				return true
			end
		end
		return false
	end,
	
	['id'] = function( link, ...)
		local iID = GetItemValue( link, 'id')
		for n = 1,select( '#', ...) do
			local ref = select( n, ...)
			if iID == GetRefValue( ref, 'id') then
				return true
			end
		end
		return false
	end,
	
	['quality'] = function( link, ...)
		local iQuality = GetItemValue( link, 'quality')
		for n = 1,select( '#', ...) do
			local ref = select( n, ...)
			if iQuality == GetRefValue( ref, 'quality') then
				return true
			end
		end
		return false
	end,
	
	['level'] = function( link, min, max)
		local iLevel = GetItemValue( link, 'level')
		local iMin = GetRefValue( min, 'level')
		local iMax = GetRefValue( max, 'level')
		if iMax > iMin then
			return iMax >= iLevel and iLevel >= iMin
		else
			return iLevel >= iMin
		end
	end,
	
	['uselevel'] = function( link, min)
		local iLevel = GetItemValue( link, 'level')
		return iLevel >= GetRefValue( min, 'level')
	end,
	
	['type'] = function( link, ...)
		local iType = GetItemValue( link, 'type')
		for n = 1,select( '#', ...) do
			local ref = select( n, ...)
			if iType == GetRefValue( ref, 'type') then
				return true
			end
		end
		return false
	end,
	
	['subtype'] = function( link, ...)
		local iSubType = GetItemValue( link, 'subtype')
		for n = 1,select( '#', ...) do
			local ref = select( n, ...)
			if iSubType == GetRefValue( ref, 'subtype') then
				return true
			end
		end
		return false
	end,
	
	['count'] = function( link, count)
		local iCount = GetItemValue( link, 'count')
		return iCount >= GetRefValue( count, 'count')
	end,
	
	['equip'] = function( link, ...)
		local iEquip = GetItemValue( link, 'equip')
		for n = 1,select( '#', ...) do
			local ref = select( n, ...)
			if iEquip == GetRefValue( ref, 'equip') then
				return true
			end
		end
		return false
	end,

	['useable'] = function( link)
		local iEquip = GetItemValue( link, 'equip')
		return iEquip and true or false
	end,

	['outfit'] = function( link, ...)
		if IsAddOnLoaded( 'Outfitter') or Outfitter:IsInitialized() then
			local info = Outfitter:GetItemInfoFromLink( link )
			if info then
				local outfits = Outfitter:GetOutfitsUsingItem( info )
				if outfits and #outfits > 0 then
					for n = 1,select( '#', ...) do
						local ref = select( n, ...)
						for _,o in pairs( outfits) do
							if o and o == ref then
								return true
							end
						end
					end
				end
			end
		end
		return false
	end,
}
--[[---------------------------------------------------------------------------------
  lib functions
------------------------------------------------------------------------------------]]
local function Link( link)
--	lib:Print( 'Link', link)
	local id = GetItemID( link)
	if id then
		return 'ref@'..id
	end
	return link
end

local function Function( fkt, params)
--	lib:Print( 'Function', fkt, params)
	params = params:gsub( '|c%x+|H[^%]]+%]|h|r', Link) -- '|c%s|Hitem:%i:0:0:0:0:0:0:0|h[%s]|h|r'
	params = params:gsub( '["\']', '')
	return fkt:lower()..'(link,'..params:gsub( '[%w@]+', '"%1"')
end

function lib:CompileRule( template)
--	self:Print( 'CompileRule', template)
	local code = tostring( template)
	code = code:gsub( '%s+', ' ')
--	self:Print( 'CompileRule1', code)
	code = code:gsub( '(%a+)%(([^%)]*)', Function)
	return 'local link = ... return ' .. code
end

function lib:ExecuteRule( compiled, link)
	self:Debug( 'ExecuteRule', compiled, link)
	local fkt, msg = loadstring( compiled, 'Error in formula' )
	if fkt then
		setfenv( fkt, Environment)
		local ok, result = pcall( fkt, link)
		if ok then
			return result
		end
--		self:Print( 'ExecuteRule', result)
	else
--		self:Print( 'ExecuteRule', msg)
	end
	return false
end

function lib:IsRuleValid( template, link)
	if template and link then
		local code = self:CompileRule( template)
		return self:ExecuteRule( code, link)
	end
	return false
end

--[[---------------------------------------------------------------------------------
  embedding and embed handling
------------------------------------------------------------------------------------]]

local mixins = {
	'CompileRule',
	'ExecuteRule',
	'IsRuleValid',
} 

function lib:Embed( target )
	for k,v in pairs( mixins ) do
		target[v] = self[v]
	end
	self.embeds[target] = true
	return target
end

for addon in pairs( lib.embeds) do
	lib:Embed( addon)
end
